import Koa from 'koa'
import dotenv from 'dotenv'
import route from 'koa-route'
import Debug from 'debug'
import koaBody from 'koa-body'
import func from './func'
import { ErrorDescEnum, HttpError } from './types/HttpError'

dotenv.config()
console.log = Debug('invoiceIssuer:app.ts')

const app = new Koa()
app.use(koaBody())

// 错误处理中间件
app.use(async (ctx, next) => {
	try {
		await next()
	} catch (e) {
		if (e instanceof HttpError) {
			ctx.status = e.status
			ctx.body = JSON.stringify({ error: e.message })
		} else {
			console.log(e)
			ctx.status = 500
			ctx.body = JSON.stringify({ error: ErrorDescEnum.unknown_issues })
		}
	}
})

app.use(route.get('/', (ctx) => {
	ctx.body = 'Hello World'
}))

app.use(route.post('/payer', async (ctx) => {
	// TODO: 请求头验证 bearer token

	// 验证必填字段
	// 字段缺失时
	if (!ctx.request.body) throw new HttpError(ErrorDescEnum.required_fields_missing, 400)

	// 有字段，但为空时
	const { name, address, email, abn } = ctx.request.body
	if (!name || !address || !email) throw new HttpError(ErrorDescEnum.required_fields_missing, 400)

	// 验证地址格式，格式应为 string[]，最多两行
	if (!Array.isArray(address) || address.length > 2 || address.length <= 0) throw new HttpError(ErrorDescEnum.invalid_field_format, 400)

	// 名字应为字符串
	if (typeof name !== 'string') throw new HttpError(ErrorDescEnum.invalid_field_format, 400)
	
	// 验证邮箱格式
	const emailRegex = /^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/
	if (!emailRegex.test(email)) throw new HttpError(ErrorDescEnum.invalid_field_format, 400)

	// 创建新的付款人
	await func.createPayer(name, address, email, abn)
	ctx.status = 204
}))

const port = parseInt(process.env.PORT ?? '3000')
app.listen(port, () => {
	console.log(`Server is running at http://localhost:${port}`)
})